"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBlankGraphPaths = void 0;
var getRGBColorString_1 = require("../util/getRGBColorString");
var _path_1 = require("../renderers/RealRenderer/strokeNodes/_path");
var _polygon_1 = require("../renderers/RealRenderer/strokeNodes/_polygon");
function getBlankGraphPaths(dimensions, bgColor, bgType) {
    var outX = dimensions[0], outY = dimensions[1];
    var axesPath = new _path_1.Path('', 'bg');
    switch (bgType.type) {
        case 'none':
            break;
        case 'axes':
            var X = Math.floor(outY * (bgType.xOffset / 100));
            var Y = Math.floor(outX * (bgType.yOffset / 100));
            axesPath.setStroke(getRGBColorString_1.getRGBColorString(bgType.axesColor));
            axesPath.appendPath("M 0," + Y + " H " + (dimensions[0] - 1));
            axesPath.appendPath("M " + X + ",0 V " + (dimensions[1] - 1));
            break;
        case 'grid':
            var xSpacing = dimensions[0] * (bgType.xSpacing / 100);
            var ySpacing = dimensions[1] * (bgType.ySpacing / 100);
            console.log(xSpacing, ySpacing);
            axesPath.setStroke(getRGBColorString_1.getRGBColorString(bgType.lineColor));
            // x-axis
            for (var x = xSpacing; x < dimensions[0]; x += xSpacing) {
                axesPath.appendPath("M " + x + ",0 V " + (dimensions[1] - 1));
            }
            // y-axis
            for (var y = ySpacing; y < dimensions[1]; y += ySpacing) {
                axesPath.appendPath("M 0," + y + " H " + (dimensions[0] - 1));
            }
            break;
        case 'ruled':
            var perpendicularDimension = bgType.orientation === 'horizontal' ? dimensions[1] : dimensions[0];
            var spacing = perpendicularDimension * (bgType.spacing / 100);
            axesPath.setStroke(getRGBColorString_1.getRGBColorString(bgType.lineColor));
            for (var dist = spacing; dist < perpendicularDimension; dist += spacing) {
                var d = void 0;
                if (bgType.orientation === 'horizontal')
                    d = "M 0," + dist + " H " + (dimensions[0] - 1);
                else
                    d = "M " + dist + ",0 V " + (dimensions[1] - 1);
                axesPath.appendPath(d);
            }
            break;
        default:
            break;
    }
    var bgPolygon = new _polygon_1.Polygon([
        [0, 0],
        [0, dimensions[1]],
        [dimensions[0], dimensions[1]],
        [dimensions[0], 0]
    ], 'bg');
    bgPolygon.setFill(getRGBColorString_1.getRGBColorString(bgColor));
    bgPolygon.setStroke(getRGBColorString_1.getRGBColorString(bgColor));
    return [bgPolygon, axesPath];
}
exports.getBlankGraphPaths = getBlankGraphPaths;
