import { RealRenderer } from '../RealRenderer/RealRenderer';
import { IRealDrawBoardEvents } from './events/RealDrawBoardEvents';
import { RealDrawBoardOptions, RealDrawBoardSettings } from '../../types/RealDrawBoardTypes';
export * as RealRendererTypes from '../../types/RealRendererTypes';
export * as RealDrawBoardTypes from '../../types/RealDrawBoardTypes';
export * from '../../constants/defaults/RealDrawBoardDefaults';
import { changeTool, changeToolSetting, clear, clearPreview, _resetBoard } from './boardManip';
import { _addDOMEvents, _removeDOMEvents } from './_DOMEvents';
import { _getMouseCoords, _getTouchCoords } from './_coords';
import { Tool, ToolSettings } from './tools/tools';
import { Coordinate, Stroke } from '../../types/RealRendererTypes';
export declare class RealDrawBoard extends RealRenderer<IRealDrawBoardEvents> {
    settings: RealDrawBoardSettings;
    tool: Tool;
    toolSettings: ToolSettings;
    /** key -> identifier, value -> coordinate
     *  For mouse, the key is 'mouse', for touches, stringified identifier -> https://developer.mozilla.org/en-US/docs/Web/API/Touch/identifier
     */
    _lastCoords: Map<string, Coordinate>;
    _doPreview: boolean;
    /**
     * The preview for the current stroke
     */
    _previewStroke: Map<string, Stroke>;
    protected _resetBoard: typeof _resetBoard;
    protected _addDOMEvents: typeof _addDOMEvents;
    protected _removeDOMEvents: typeof _removeDOMEvents;
    protected _startStroke: typeof import("./tools/brush")._startStroke | typeof import("./tools/eraser")._startStroke | typeof import("./tools/line")._startStroke;
    protected _endStroke: typeof import("./tools/brush")._endStroke | typeof import("./tools/eraser")._endStroke | typeof import("./tools/line")._endStroke;
    protected _doStroke: typeof import("./tools/brush")._doStroke | typeof import("./tools/eraser")._doStroke | typeof import("./tools/line")._doStroke;
    protected _toolPreview: typeof import("./tools/brush")._toolPreview | typeof import("./tools/eraser")._toolPreview | typeof import("./tools/line")._toolPreview;
    protected _onScroll: typeof import("./tools/brush")._onScroll | typeof import("./tools/eraser")._onScroll | typeof import("./tools/line")._onScroll;
    protected _getMouseCoords: typeof _getMouseCoords;
    protected _getTouchCoords: typeof _getTouchCoords;
    changeToolSetting: typeof changeToolSetting;
    changeTool: typeof changeTool;
    clearPreview: typeof clearPreview;
    clear: typeof clear;
    constructor(options: RealDrawBoardOptions);
    protected _mouseDownEventListener: (e: MouseEvent) => void;
    protected _mouseUpEventListener: (e: MouseEvent) => void;
    protected _mouseLeaveEventListener: (e: MouseEvent) => void;
    protected _mouseMoveEventListener: (e: MouseEvent) => void;
    protected _previewMouseMoveEventListener: (e: MouseEvent) => void;
    protected _wheelEventListener: (e: WheelEvent) => void;
    protected _panEventListener: (e: MouseEvent) => void;
    protected _touchStartEventListener: (e: TouchEvent) => void;
    protected _touchEndEventListener: (e: TouchEvent) => void;
    protected _touchMoveEventListener: (e: TouchEvent) => void;
    protected _previewTouchMoveEventListener: (e: TouchEvent) => void;
    startRender(): this;
    stopRender(): this;
    reset(): this;
}
