"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports._resetBoard = exports.clear = exports.clearPreview = exports.changeToolSetting = exports.changeTool = void 0;
var tools_1 = require("./tools/tools");
var blankGraph_1 = require("../../pathMakers/blankGraph");
function changeTool(newTool) {
    var oldTool = this.tool;
    this.tool = newTool;
    this._startStroke = tools_1.tools[this.tool]._startStroke;
    this._doStroke = tools_1.tools[this.tool]._doStroke;
    this._endStroke = tools_1.tools[this.tool]._endStroke;
    this._toolPreview = tools_1.tools[this.tool]._toolPreview;
    this._onScroll = tools_1.tools[this.tool]._onScroll;
    this._previewStroke.forEach(function (stroke) {
        stroke.forEach(function (node) {
            node.delete();
        });
    });
    this._previewStroke.clear();
    this.emit('tool-change', {
        newTool: newTool,
        oldTool: oldTool
    });
    return this;
}
exports.changeTool = changeTool;
function changeToolSetting(settingName, value) {
    var oldValue = this.toolSettings[settingName];
    this.toolSettings[settingName] = value;
    this.emit('tool-setting-change', {
        settingName: settingName,
        newValue: value,
        oldValue: oldValue
    });
    return this;
}
exports.changeToolSetting = changeToolSetting;
function clearPreview() {
    this._previewStroke.forEach(function (stroke) {
        stroke.forEach(function (node) {
            node.delete();
        });
    });
    this._previewStroke.clear();
    return this;
}
exports.clearPreview = clearPreview;
function clear() {
    if (this._strokeIndex > 0) {
        this._strokeIndex = 0;
        this._lastCoords.clear();
        this.strokes.forEach(function (stroke) {
            stroke.forEach(function (strokeNode) { return strokeNode.delete(); });
        });
        this.strokes = [
            blankGraph_1.getBlankGraphPaths(this.dimensions, this.bgColor, this.bgType)
        ];
        this._display(this.strokes[this._strokeIndex]);
        this.emit('board-cleared', {});
    }
    return this;
}
exports.clear = clear;
function _resetBoard() {
    this.bgColor = this.settings.bgColor;
    this.tool = this.settings.tool;
    this.toolSettings = __assign(__assign({}, tools_1.ToolDefaults), this.settings.toolSettings);
    this._lastCoords.clear();
    this.stopRender();
    this.emit('board-reset', {});
    return this;
}
exports._resetBoard = _resetBoard;
