import { EventEmitter } from './events/eventEmitter';
import { IRealRendererEvents } from './events/RealRendererEvents';
import { GraphDimensions, Color, RealRendererOptions, Stroke, RealRendererSettings, BGType, RealExport, SVGSections } from '../../types/RealRendererTypes';
export * as RealRendererTypes from '../../types/RealRendererTypes';
export * from '../../constants/defaults/RealRendererDefaults';
import { undo, redo } from './undo';
export declare class RealRenderer<EventTypes extends IRealRendererEvents = IRealRendererEvents> extends EventEmitter<EventTypes> {
    svg: SVGSVGElement;
    svgSections: SVGSections;
    dimensions: GraphDimensions;
    scaleFactor: number;
    protected _offsetX: number;
    protected _offsetY: number;
    originalDimensions: GraphDimensions;
    strokes: Stroke[];
    settings: RealRendererSettings;
    protected _strokeIndex: number;
    bgColor: Color;
    bgType: BGType;
    drawsPerFrame: number;
    timeStep: number;
    time: number;
    protected _doRender: boolean;
    undo: typeof undo;
    redo: typeof redo;
    constructor(options: RealRendererOptions, eventList?: (keyof EventTypes)[]);
    protected _setViewBox(dimensions: GraphDimensions, xOffset: number, yOffset: number): void;
    protected _drawFunc(time: number): this;
    protected _draw(): this;
    protected _addStroke(stroke: Stroke): void;
    draw(numDraws?: number): this;
    scale(scaleFactor: number): void;
    changeOffsets(xOffset: number, yOffset: number): void;
    protected _render(): void;
    _display(stroke: Stroke): void;
    startRender(): this;
    stopRender(): this;
    toggleRender(): this;
    exportData(): RealExport;
    importData(data: RealExport): void;
    resetTime(): this;
    reset(): this;
}
